@@ -1,65 +1,28 @@
 POSITIVE.PARAMETERS <- c("major","minor","tau position","tau velocity","tau","omega","error")
 
-# clean up parameter arrays
-clean.parameters <- function(par,linear.cov=FALSE)
-{
-  NAMES <- names(par)
-
-  for(P in POSITIVE.PARAMETERS) { if(P %in% NAMES) { par[P] <- clamp(par[P],0,Inf) } }
-
-  # covariance parameters
-  if("minor" %in% NAMES)
-  {
-    if(!linear.cov)
-    {
-      # swap major and minor axis --- if not profiling major axis
-      if('major' %in% NAMES && par["minor"]>par['major'])
-      {
-        par["angle"] <- par["angle"] - pi/2
-
-        VARS <- par[c('minor','major')]
-        par['major'] <- max(VARS)
-        par['minor'] <- min(VARS)
-      }
-
-      # wrap angle back to (-pi/2,+pi/2)
-      par["angle"] <- (((par["angle"]/pi+1/2) %% 1) - 1/2)*pi
-    }
-    else
-    {
-      # sigma[x,x] sigma[y,y] --- positive semi-definite
-      MAX <- sqrt(prod(par[c('major','minor')]))
-      # sigma[x,y]
-      par['angle'] <- clamp(par['angle'],-MAX,MAX)
-    }
-  }
-
-  P <- c("tau position","tau velocity")
-  if(all(P %in% NAMES)) { par[P] <- sort(par[P],decreasing=TRUE) }
-
-  return(par)
-}
-
 
 # identify autocovariance parameters in ctmm object
-# if profile==TRUE, some parameters can be solved exactly and so aren't identified
+# if profile==TRUE, some parameters can be solved exactly and so aren't identified -- in particular, only relative variances matter
 # if linear==TRUE, only return linear non-problematic parameters
-# if linear.cov==TRUE, use linear covariance paramters for pREML
+# if linear.cov==TRUE, use linear covariance parameters for pREML
 # STRUCT determines the model structure incase some estimated parameters in CTMM are zero
-id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE=FALSE,dt=0,df=0,dz=0,STRUCT=CTMM)
+id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE.FIT=NA,dt=0,df=0,dz=0,STRUCT=CTMM)
 {
+  AXES <- length(CTMM$axes)
   # identify and name autocovariance parameters
   NAMES <- NULL
 
   if(STRUCT$sigma@par['major'] || length(STRUCT$tau))
   {
     sigma <- attr(CTMM$sigma,"par")
-    if(!profile || (STRUCT$error && UERE>=3)) # must fit all 1-3 covariance parameters
+    if(!profile || any(STRUCT$error & !UERE.FIT)) # must fit all 1-3 covariance parameters - not profiling or fixed errors
     { if(STRUCT$isotropic) { NAMES <- c(NAMES,"major") } else { NAMES <- c(NAMES,names(sigma)) } }
-    else if(STRUCT$error || STRUCT$circle) # must fit shape, but scale/area/var can be profiled for free
+    else if(any(UERE.FIT) || STRUCT$circle) # must fit shape, but overall scale/var can be profiled for free - circulation or relative errors
     { if(!STRUCT$isotropic) { NAMES <- c(NAMES,names(sigma[2:3])) } }
   }
 
+  if(any(UERE.FIT)) { NAMES <- c(NAMES, paste("error",names(UERE.FIT)[UERE.FIT>0]) ) }
+
   if(!linear) # nonlinear autocorrelation parameters
   {
     TAU <- STRUCT$tau # for structure here
@@ -78,7 +41,7 @@ id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE=F
     if(STRUCT$circle) { NAMES <- c(NAMES,"circle") }
   }
 
-  if((STRUCT$error) && UERE<3) { NAMES <- c(NAMES,"error") }
+  ## ORDER MATTERS BETWEEN THESE TWO CODE BLOCKS!
 
   # setup parameter information
   parscale <- NULL
@@ -110,6 +73,15 @@ id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE=F
       upper <- c(upper,Inf,Inf)
       period <- c(period,FALSE,pi)
     }
+
+    # RMS UERE - error parameters
+    if(any(UERE.FIT))
+    {
+      parscale <- c(parscale, pmax(CTMM$error[UERE.FIT],dz) ) # minimum of dz error parscale
+      lower <- c(lower, rep(0,sum(UERE.FIT)) )
+      upper <- c(upper, rep(Inf,sum(UERE.FIT)) )
+      period <- c(period, rep(FALSE,sum(UERE.FIT)) )
+    }
   }
   else # linear sigma: xx, yy, xy
   {
@@ -128,6 +100,15 @@ id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE=F
       upper <- c(upper,Inf,Inf)
       period <- c(period,FALSE,FALSE)
     }
+
+    # MS UERE - error parameters
+    if(any(UERE.FIT))
+    {
+      parscale <- c(parscale, pmax(CTMM$error[UERE.FIT],dz)^2 ) # minimum of dz error parscale
+      lower <- c(lower, rep(0,sum(UERE.FIT)) )
+      upper <- c(upper, rep(Inf,sum(UERE.FIT)) )
+      period <- c(period, rep(FALSE,sum(UERE.FIT)) )
+    }
   } # end linear sigma
 
   if(!linear) # nonlinear autocorrelation parameters
@@ -157,17 +138,6 @@ id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE=F
     }
   }
 
-  if(STRUCT$error && UERE<3)
-  {
-    # relative length scale
-    # SCALE <- ifelse(profile,sqrt(CTMM$sigma@par['major']),1)
-
-    parscale <- c(parscale,max(dz,CTMM$error)) # minimum of dz error parscale
-    lower <- c(lower,0)
-    upper <- c(upper,Inf)
-    period <- c(period,FALSE)
-  }
-
   if(length(parscale))
   {
     names(parscale) <- NAMES
@@ -176,13 +146,35 @@ id.parameters <- function(CTMM,profile=TRUE,linear=FALSE,linear.cov=FALSE,UERE=F
     names(period) <- NAMES
   }
 
+  # only relative variance matters - easier optimization (more constrained, well behaved)
+  if(profile)
+  {
+    PARS <- c("major","minor")
+    PARS <- PARS[PARS %in% NAMES]
+
+    if(length(PARS))
+    {
+      parscale[PARS] <- 1
+      upper[PARS] <- 1
+    }
+
+    # error fraction could be much smaller
+    if(any(UERE.FIT))
+    {
+      PARS <- paste("error",names(UERE.FIT)[UERE.FIT])
+
+      parscale[PARS] <- min(1, pmax(CTMM$error[UERE.FIT],dz) / sqrt(MAX) )
+      upper[PARS] <- 1
+    }
+  } # end profile
+
   return(list(NAMES=NAMES,parscale=parscale,lower=lower,upper=upper,period=period))
 }
 
 
 # pull out array of named autocovariance parameters
 # linear.cov uses linear representation of covariance matrix
-get.parameters <- function(CTMM,NAMES,linear.cov=FALSE)
+get.parameters <- function(CTMM,NAMES,profile=FALSE,linear.cov=FALSE)
 {
   par <- numeric(length(NAMES))
   names(par) <- NAMES
@@ -197,12 +189,18 @@ get.parameters <- function(CTMM,NAMES,linear.cov=FALSE)
     getter("major",sigma[1])
     getter("minor",sigma[2])
     getter("angle",sigma[3])
+
+    PARS <- NAMES[ grepl("error",NAMES) ]
+    for(P in PARS) { par[P] <- CTMM$error[ substr(P,nchar("error ?"),nchar(P)) ] }
   }
   else
   {
     getter("major",sigma[1]) # sigma[x,x]
     getter("minor",sigma[4]) # sigma[y,y]
     getter("angle",sigma[2]) # sigma[x,y]
+
+    PARS <- NAMES[ grepl("error",NAMES) ]
+    for(P in PARS) { par[P] <- CTMM$error[ substr(P,nchar("error ?"),nchar(P)) ]^2 }
   }
 
   tau <- CTMM$tau
@@ -212,33 +210,149 @@ get.parameters <- function(CTMM,NAMES,linear.cov=FALSE)
   getter("omega")
 
   getter("circle")
-  getter("error")
+
+  # only relative variances matter - return variance fraction [0,1]
+  if(profile)
+  {
+    UERE.RMS <- paste("error",names(CTMM$error)) %in% PARS
+    UERE.RMS <- CTMM$error[UERE.RMS]
+
+    MAX <- profiled.var(CTMM,UERE.RMS=UERE.RMS,AVE=FALSE)
+
+    # this needs to be consistent with profiled.var() and set.parameters()
+    if(length(PARS))
+    { par[PARS] <- par[PARS] / sqrt(MAX) }
+
+    SPARS <- c("major","minor")
+    SPARS <- SPARS[SPARS %in% NAMES]
+
+    # this needs to be consistent with profiled.var() and set.parameters()
+    if(length(SPARS)) { par[SPARS] <- par[SPARS] / MAX }
+  } # end profile
+
+  return(par)
+}
+
+
+# total variance to profile
+profiled.var <- function(CTMM,sigma=CTMM$sigma,UERE.RMS=CTMM$error,DT=1,AVE=FALSE)
+{
+  PRO.VAR <- sum(eigenvalues.covm(sigma))*DT # really profiling the variance with mean?
+
+  if(any(UERE.RMS>0)) { PRO.VAR <- PRO.VAR + sum(UERE.RMS^2) } # comparable error variance (@DOP==1)
+
+  if(AVE) # variance per dimension
+  {
+    AXES <- length(CTMM$axes)
+    PRO.VAR <- PRO.VAR / AXES
+  }
+
+  return(PRO.VAR)
+}
+
+
+# clean up parameter arrays
+clean.parameters <- function(par,profile=FALSE,linear.cov=FALSE)
+{
+  NAMES <- names(par)
+
+  for(P in POSITIVE.PARAMETERS) { if(P %in% NAMES) { par[P] <- clamp(par[P],0,Inf) } }
+
+  # arbitrary error parameters > 0
+  EP <- NAMES[ grepl("error",NAMES) ]
+  par[EP] <- clamp(par[EP],0,Inf)
+
+  # make sure relative variance is constrained properly
+  if(profile)
+  {
+    SP <- c('major','minor') # major can't really be here
+    SP <- SP[SP %in% NAMES]
+
+    # 0 <= VBOUND <= 1
+    VBOUND <- sum(par[SP]) + sum(par[EP]^2)
+    if(VBOUND>1)
+    {
+      if(length(EP)) { par[EP] <- par[EP] / sqrt(VBOUND) }
+
+      if(length(SP))
+      {
+        # par[SP] <- par[SP] / VBOUND # new major
+        par['minor'] <- 0 # old major
+        par["angle"] <- par["angle"] - pi/2
+      }
+    } # else 'major' > 0
+    else if(length(SP) && VBOUND<par['minor'])
+    {
+      par['minor'] <- VBOUND # old major is true minor
+      par["angle"] <- par["angle"] - pi/2 # rotate old major to true major
+    }
+  }
+  else if(!linear.cov && all(c("major","minor") %in% NAMES) && par["minor"]>par['major'])
+  {
+    # swap major and minor axis --- if not profiling major axis
+    VARS <- par[c('minor','major')]
+    names(VARS) <- NULL
+    par[c('major','minor')] <- VARS
+
+    par["angle"] <- par["angle"] - pi/2
+  }
+  else if(linear.cov && all(c("major","minor") %in% NAMES))
+  {
+    # sigma[x,x] sigma[y,y] --- positive semi-definite
+    MAX <- sqrt(prod(par[c('major','minor')]))
+    # sigma[x,y]
+    par['angle'] <- clamp(par['angle'],-MAX,MAX)
+  }
+
+  # shift back to (-pi/2,pi/2)
+  if("angle" %in% NAMES && !linear.cov)
+  { par["angle"] <- (((par["angle"]/pi+1/2) %% 1) - 1/2)*pi }
+
+  # swap tau velocity and tau position if they become reversed (to keep parscale sane)
+  P <- c("tau position","tau velocity")
+  if(all(P %in% NAMES)) { par[P] <- sort(par[P],decreasing=TRUE) }
 
   return(par)
 }
 
 
 # write autocovariance parameter array into a CTMM object
-set.parameters <- function(CTMM,par,linear.cov=FALSE,optimize=FALSE)
+set.parameters <- function(CTMM,par,profile=FALSE,linear.cov=FALSE)
 {
   NAMES <- names(par)
+  AXES <- length(CTMM$axes)
 
   sigma <- CTMM$sigma
   if(!linear.cov)
   {
     sigma <- attr(sigma,"par")
-    NAME <- "major"; if(NAME %in% NAMES) { sigma[NAME] <- par[NAME]; if(length(CTMM$axes)>1) { sigma['minor'] <- par[NAME] } } # incase of isotropic
+    NAME <- "major"; if(NAME %in% NAMES) { sigma[NAME] <- par[NAME]; if(AXES>1) { sigma['minor'] <- par[NAME] } } # in case of isotropic
     NAME <- "minor"; if(NAME %in% NAMES) { sigma[NAME] <- par[NAME] }
     NAME <- "angle"; if(NAME %in% NAMES) { sigma[NAME] <- par[NAME] }
+
+    EP <- NAMES[ grepl("error",NAMES) ]
+    for(P in EP) { CTMM$error[ substr(P,nchar("error ?"),nchar(P)) ] <- par[P] }
+
+    if(profile)
+    {
+      SP <- c("major","minor") # major shouldn't be here
+      SP <- SP[SP %in% NAMES]
+
+      MAX <- sum(par[SP]) + sum(par[EP]^2)
+      sigma['major'] <- 1 - MAX # remaining fraction of total variance
+    }
   }
-  else
+  else # major,minor,angle is storing xx,yy,xy in linear representation # no profiling ever here
   {
     sigma <- methods::getDataPart(sigma)
-    NAME <- "major"; if(NAME %in% NAMES) { diag(sigma) <- par[c(NAME,NAME)] }
+    NAME <- "major"; if(NAME %in% NAMES) { if(AXES>1) { diag(sigma) <- par[c(NAME,NAME)] } else { sigma <- par[NAME] } }
     NAME <- "minor"; if(NAME %in% NAMES) { sigma[4] <- par[NAME] }
     NAME <- "angle"; if(NAME %in% NAMES) { sigma[2:3] <- par[c(NAME,NAME)] }
+
+    PARS <- NAMES[ grepl("error",NAMES) ]
+    for(P in PARS) { CTMM$error[ substr(P,nchar("error ?"),nchar(P)) ] <- sqrt(par[P]) }
   }
-  CTMM$sigma <- covm(sigma)
+  CTMM$sigma <- covm(sigma,axes=CTMM$axes)
 
   NAME <- "tau position"; if(NAME %in% NAMES) { CTMM$tau[1] <- par[NAME] ; names(CTMM$tau)[1] <- 'position' }
   NAME <- "tau velocity"; if(NAME %in% NAMES) { CTMM$tau[2] <- par[NAME] ; names(CTMM$tau)[2] <- 'velocity' }
@@ -246,7 +360,6 @@ set.parameters <- function(CTMM,par,linear.cov=FALSE,optimize=FALSE)
   NAME <- "omega"; if(NAME %in% NAMES) { CTMM$omega <- par[NAME] }
 
   NAME <- "circle"; if(NAME %in% NAMES) { CTMM[[NAME]] <- par[NAME] }
-  NAME <- "error"; if(NAME %in% NAMES) { CTMM[[NAME]] <- par[NAME] }
 
   return(CTMM)
 }
@@ -274,7 +387,13 @@ copy.parameters <- function(x,value,par=value$features,destructive=TRUE)
   if("tau" %in% Pv) { x$tau <- value$tau }
   if("omega" %in% Pv) { x$omega <- value$omega }
   if("circle" %in% Pv) { x$circle <- value$circle }
-  if("error" %in% Pv) { x$error <- value$error }
+
+  PARS <- Pv[ grepl("error",Pv) ]
+  for(P in PARS)
+  {
+    P <- substr(P,nchar("error ?"),nchar(P))
+    x$error[P] <- value$error[P]
+  }
 
   # don't think I need this
   if("MLE" %in% names(x))
